import 'package:fish_redux/fish_redux.dart';
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_plugin/live_push_config.dart';

enum CameraPushPagePreviewState {
  startPreview,
  stopPreview,
}

enum CameraPushPagePushState {
  startPush,
  stopPush,
  pausePush,
}

enum CameraPushPageBottomSheetState {
  showMusic,
  showMorSetting,
  nothing,
}

///bgm
class BGMState implements Cloneable<BGMState> {
  bool isOpenBGMEarsBack = false;
  bool isOpenAudioDenoise = false;
  bool isBGMMute = false;
  bool isBGMLoop = false;
  bool isBGMPlaying = false;
  int bgmVolume = 50;
  int bgmCaptureVolume = 50;
  int bgmSelectedIndex = 0;
  int bgmProgress = 0;
  int bgmDuration = 0;

  @override
  BGMState clone() => BGMState()
    ..bgmVolume = bgmVolume
    ..bgmCaptureVolume = bgmCaptureVolume
    ..isOpenBGMEarsBack = isOpenBGMEarsBack
    ..isOpenAudioDenoise = isOpenAudioDenoise
    ..isBGMMute = isBGMMute
    ..isBGMLoop = isBGMLoop
    ..isBGMPlaying = isBGMPlaying
    ..bgmSelectedIndex = bgmSelectedIndex
    ..bgmProgress = bgmProgress
    ..bgmDuration = bgmDuration;
}

///more setting
class PushSettingState implements Cloneable<PushSettingState> {
  bool isOpenPushMirror = false;
  bool isOpenPreviewMirror = false;
  int videoTargetBitrate = 1400;
  int videoMinBitrate = 600;
  int previewModelIndex = 1;

  @override
  PushSettingState clone() => PushSettingState()
    ..isOpenPushMirror = isOpenPushMirror
    ..isOpenPreviewMirror = isOpenPreviewMirror
    ..videoTargetBitrate = videoTargetBitrate
    ..videoMinBitrate = videoMinBitrate
    ..previewModelIndex = previewModelIndex;
}

class CameraPushState implements Cloneable<CameraPushState> {
  String pushURL = "";
  bool isOpenFlashLamp = false;
  AlivcLivePusherConfig pusherConfig = AlivcLivePusherConfig.init();
  bool beautyOn = false;
  bool showBeautyMenu = false;
  String pushStatusTip = "";

  BGMState bgmState = BGMState();
  PushSettingState pushSettingState = PushSettingState();

  CameraPushPageBottomSheetState bottomSheetState =
      CameraPushPageBottomSheetState.nothing;

  CameraPushPagePreviewState previewState =
      CameraPushPagePreviewState.startPreview;
  CameraPushPagePushState pushState = CameraPushPagePushState.stopPush;

  @override
  CameraPushState clone() => CameraPushState()
    ..pushURL = pushURL
    ..beautyOn = beautyOn
    ..showBeautyMenu = showBeautyMenu
    ..bgmState = bgmState
    ..pushSettingState = pushSettingState
    ..bottomSheetState = bottomSheetState
    ..previewState = previewState
    ..pushState = pushState
    ..pushStatusTip = pushStatusTip;
}

CameraPushState initState(Map<String, dynamic> args) {
  return CameraPushState()
    ..pushURL = args[pushURLKey]
    ..pusherConfig = args[pushConfigKey]
    ..beautyOn = args[pushBeautyOnKey];
}
