import 'dart:math';

import 'package:fish_redux/fish_redux.dart';
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_enrollment/interactive_enrollment_state.dart';
import 'interactive_enrollment_action.dart';

Reducer<InteractiveEnrollmentState> buildReducer() {
  return asReducer(
    <Object, Reducer<InteractiveEnrollmentState>>{
      InteractiveEnrollmentAction.onChangeIdentity: _onChangeIdentity,
      InteractiveEnrollmentAction.onChangeAppInfoState: _onChangeAppInfoState,
      InteractiveEnrollmentAction.onEditAppInfo: _onEditAppInfo,
      InteractiveEnrollmentAction.onChangeUserId: _onChangeUserId,
      InteractiveEnrollmentAction.onChangeRoomId: _onChangeRoomId,
    },
  );
}

InteractiveEnrollmentState _onChangeIdentity(
    InteractiveEnrollmentState state, Action action) {
  final InteractiveEnrollmentState newState = state.clone();
  newState.interactiveIdentityTypeEnum =
      action.payload ?? InteractiveIdentityTypeEnum.anchor;
  return newState;
}

InteractiveEnrollmentState _onChangeAppInfoState(
    InteractiveEnrollmentState state, Action action) {
  final InteractiveEnrollmentState newState = state.clone();
  bool showAppInfo = action.payload ?? false;
  newState.showAppInfo = showAppInfo;
  if (showAppInfo) {
    newState.arrowAngle = pi;
  } else {
    newState.arrowAngle = 0;
  }
  return newState;
}

InteractiveEnrollmentState _onEditAppInfo(
    InteractiveEnrollmentState state, Action action) {
  final InteractiveEnrollmentState newState = state.clone();
  Map<String, dynamic>? map = action.payload;
  if (map != null) {
    newState.appId = map[interactiveAppIdKey];
    newState.appKey = map[interactiveAppKeyKey];
    newState.playDomain = map[interactivePlayDomainKey];
  }
  return newState;
}

InteractiveEnrollmentState _onChangeUserId(
    InteractiveEnrollmentState state, Action action) {
  final InteractiveEnrollmentState newState = state.clone();
  newState.userId = action.payload ?? "";
  return newState;
}

InteractiveEnrollmentState _onChangeRoomId(
    InteractiveEnrollmentState state, Action action) {
  final InteractiveEnrollmentState newState = state.clone();
  newState.roomId = action.payload ?? "";
  return newState;
}
