import 'package:fish_redux/fish_redux.dart';
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter/cupertino.dart' hide Action;
import 'package:flutter_livepush_demo/common/InteractiveUrlUtils.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_enrollment/interactive_enrollment_page.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_info/interactive_info_state.dart';
import 'interactive_info_action.dart';

Effect<InteractiveInfoState> buildEffect() {
  return combineEffects(<Object, Effect<InteractiveInfoState>>{
    InteractiveInfoAction.clickNext: _clickNext,
    InteractiveInfoAction.clickGoBack: _clickGoBack,
    InteractiveInfoAction.changeAppId: _changeAppId,
    InteractiveInfoAction.changeAppKey: _changeAppKey,
    InteractiveInfoAction.changePlayDomain: _changePlayDomain,
  });
}

Future<void> _clickNext(
    Action action, Context<InteractiveInfoState> ctx) async {
  var interactiveInfoState = ctx.state;

  CommomUtils.saveLocal(interactiveAppIdKey, interactiveInfoState.appId);
  CommomUtils.saveLocal(interactiveAppKeyKey, interactiveInfoState.appKey);
  CommomUtils.saveLocal(interactivePlayDomainKey, interactiveInfoState.playDomain);

  var map = {
    interactiveTypeKey: interactiveInfoState.interactiveTypeEnum.name,
    interactiveAppIdKey: interactiveInfoState.appId,
    interactiveAppKeyKey: interactiveInfoState.appKey,
    interactivePlayDomainKey: interactiveInfoState.playDomain,
  };

  if (interactiveInfoState.fromHomePage) {
    Navigator.of(ctx.context).pushReplacement(
      CupertinoPageRoute(
          builder: (_) => InteractiveEnrollmentPage().buildPage(map)),
    );
  } else {
    Navigator.of(ctx.context).pop(map);
  }
}

Future<void> _clickGoBack(
    Action action, Context<InteractiveInfoState> ctx) async {
  FocusScope.of(ctx.context).unfocus();
  Navigator.of(ctx.context).pop();
}

Future<void> _changeAppId(
    Action action, Context<InteractiveInfoState> ctx) async {
  ctx.dispatch(
      InteractiveInfoActionCreator.onChangeAppId(action.payload ?? ""));
}

Future<void> _changeAppKey(
    Action action, Context<InteractiveInfoState> ctx) async {
  ctx.dispatch(
      InteractiveInfoActionCreator.onChangeAppKey(action.payload ?? ""));
}

Future<void> _changePlayDomain(
    Action action, Context<InteractiveInfoState> ctx) async {
  ctx.dispatch(
      InteractiveInfoActionCreator.onChangePlayDomain(action.payload ?? ""));
}
