import 'package:fish_redux/fish_redux.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_action.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_state.dart';

Reducer<ConfigState> buildReducer() {
  return asReducer(
    <Object, Reducer<ConfigState>>{
      ConfigAction.updateConfigData: _updateConfigData,
    },
  );
}

ConfigState _updateConfigData(ConfigState state, Action action) {
  final ConfigState newState = state.clone();
  ConfigAction actionType = action.payload.actionType;
  dynamic value = action.payload.value;
  switch (actionType) {
    case ConfigAction.updatePushURL:
      newState.configData.pushURL = value;
      break;
    case ConfigAction.selectSegmentIndex:
      newState.configData.selectSegmentIndex = value;
      break;
    case ConfigAction.setLivePushMode:
      newState.configData.livePushMode = value;
      break;
    case ConfigAction.setResolution:
      newState.configData.resolution = value;
      newState.configData.updateVideoBitrateValueWithFixQualityMode(
          newState.configData.qualityMode.intV);
      break;
    case ConfigAction.setEnableAutoBitrate:
      newState.configData.enableAutoBitrate = value;
      break;
    case ConfigAction.setEnableAutoResolution:
      newState.configData.enableAutoResolution = value;
      break;
    case ConfigAction.openAdvanced:
      newState.configData.advanced = value;
      break;
    case ConfigAction.setQualityMode:
      newState.configData.qualityMode.updateValue(value);
      newState.configData.updateVideoBitrateValueWithFixQualityMode(value);
      break;
    case ConfigAction.setTargetVideoBitrate:
      newState.configData.targetVideoBitrate = value;
      break;
    case ConfigAction.setMinVideoBitrate:
      newState.configData.minVideoBitrate = value;
      break;
    case ConfigAction.setInitialVideoBitrate:
      newState.configData.initialVideoBitrate = value;
      break;
    case ConfigAction.setAudioBitrate:
      newState.configData.audioBitrate = value;
      break;
    case ConfigAction.setMinFps:
      newState.configData.minFps = value;
      break;
    case ConfigAction.setFps:
      newState.configData.fps = value;
      break;
    case ConfigAction.setAudioSampleRate:
      newState.configData.audioSampleRate = value;
      break;
    case ConfigAction.setVideoEncodeGop:
      newState.configData.videoEncodeGop = value;
      break;
    case ConfigAction.setAudioEncoderProfile:
      newState.configData.audioEncoderProfile.updateValue(value);
      break;
    case ConfigAction.setAudioChannel:
      newState.configData.audioChannel.updateValue(value);
      break;
    case ConfigAction.setAudioEncoderMode:
      newState.configData.audioEncoderMode = value;
      break;
    case ConfigAction.setVideoEncoderMode:
      newState.configData.videoEncoderMode = value;
      break;
    case ConfigAction.setVideoHardEncoderCodec:
      newState.configData.videoHardEncoderCodec.updateValue(value);
      break;
    case ConfigAction.setOpenBFrame:
      newState.configData.openBFrame = value;
      break;
    case ConfigAction.setOrientation:
      newState.configData.orientation.updateValue(value);
      break;
    case ConfigAction.setPreviewDisplayMode:
      newState.configData.previewDisplayMode.updateValue(value);
      break;
    case ConfigAction.setConnectRetryInterval:
      newState.configData.connectRetryInterval = value;
      break;
    case ConfigAction.setConnectRetryCount:
      newState.configData.connectRetryCount = value;
      break;
    case ConfigAction.setPushMirror:
      newState.configData.pushMirror = value;
      break;
    case ConfigAction.setPreviewMirror:
      newState.configData.previewMirror = value;
      break;
    case ConfigAction.setCameraType:
      newState.configData.cameraType = value;
      break;
    case ConfigAction.setAutoFocus:
      newState.configData.autoFocus = value;
      break;
    case ConfigAction.setBeautyOn:
      newState.configData.beautyOn = value;
      break;
    case ConfigAction.setPauseImg:
      newState.configData.openPauseImg = value;
      break;
    case ConfigAction.setNetworkPoorImg:
      newState.configData.openNetworkPoorImg = value;
      break;
    case ConfigAction.setPushStreamingMode:
      newState.configData.pushStreamingMode.updateValue(value);
      break;
    case ConfigAction.openLog:
      newState.configData.openLog = value;
      break;
    default:
      break;
  }
  return newState;
}
