import 'package:fish_redux/fish_redux.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_action.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_state.dart';
import 'package:flutter/material.dart';
import 'package:flutter_livepush_demo/view/config_inputurl_view.dart';
import 'package:flutter_livepush_demo/view/config_segmentbar.dart';
import 'package:flutter_livepush_demo/view/config_cell.dart';
import 'package:flutter_livepush_demo/view/push_button.dart';
import 'package:flutter_livepush_demo/manager/push_config_manager.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flutter_livepush_demo/widgets/interactive_input_appbar.dart';
import 'package:flutter_livepush_plugin/live_push_def.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

Widget buildView(
  ConfigState state,
  Dispatch dispatch,
  ViewService viewService,
) {
  return FlutterEasyLoading(
    child: Scaffold(
      appBar: _headerRenderView(state, dispatch, viewService),
      body: _bodyRenderView(state, dispatch, viewService),
    ),
  );
}

AppBar _headerRenderView(
    ConfigState state, Dispatch dispatch, ViewService viewService) {
  if (state.pageType == PushConfigPageType.basic) {
    return AppBar(
      backgroundColor: Colors.black,
      leading: IconButton(
        icon: Image.asset(
          "assets/images/ic_back.png",
          width: 32,
          height: 32,
        ),
        onPressed: () {
          dispatch(ConfigActionCreator.goBackPage());
        },
      ),
      title: ConfigInputURLView(
        placeholder: AppLocalizations.of(viewService.context)!.config_input_url,
        defaultText: state.configData.pushURL,
        action: (value) {
          ConfigDataUpdatePayload payload =
              ConfigDataUpdatePayload.create(ConfigAction.updatePushURL, value);
          dispatch(ConfigActionCreator.updateConfigData(payload));
        },
      ),
    );
  } else {
    return buildAppBar(
      AppLocalizations.of(viewService.context)!.config_page_theme,
      leadingClickListener: () {
        dispatch(ConfigActionCreator.goBackPage());
      },
    );
  }
}

Widget _bodyRenderView(
    ConfigState state, Dispatch dispatch, ViewService viewService) {
  final _screenSize = MediaQuery.of(viewService.context).size;
  const double _segmentBarHeight = 44;
  const double _listViewLeftGap = 16;
  const double _buttomPushButtonHeight = 48;
  const double _buttomPushButtonLeftGap = 20;
  const double _verticalGap = 8;
  const double _lineHeight = 1;

  Widget segmentContainer = Container(
    margin: const EdgeInsets.only(top: _verticalGap),
    child: ConfigSegmentBar(
      item: segmentItems,
      fontSize: 16,
      selectIndex: state.configData.selectSegmentIndex,
      action: (value) {
        ConfigDataUpdatePayload payload = ConfigDataUpdatePayload.create(
            ConfigAction.selectSegmentIndex, value);
        dispatch(ConfigActionCreator.updateConfigData(payload));
      },
    ),
  );

  Widget renderListView = Container(
    margin: const EdgeInsets.only(
        top: _verticalGap + _segmentBarHeight + _lineHeight,
        left: _listViewLeftGap,
        right: _listViewLeftGap,
        bottom: _buttomPushButtonHeight + _verticalGap * 2),
    child: ListView(
      scrollDirection: Axis.vertical,
      children: _containerRenderView(state, dispatch, viewService),
    ),
  );

  Widget pushButton = Positioned(
    left: _buttomPushButtonLeftGap,
    bottom: _verticalGap + MediaQuery.of(viewService.context).padding.bottom,
    width: _screenSize.width - _buttomPushButtonLeftGap * 2,
    height: _buttomPushButtonHeight,
    child: PushButton(
      title: state.pageType == PushConfigPageType.basic
          ? AppLocalizations.of(viewService.context)!.config_push_button_basic
          : AppLocalizations.of(viewService.context)!
              .config_push_button_interactive,
      action: () {
        dispatch(ConfigActionCreator.clickPushPage(state.configData.pushURL));
      },
    ),
  );

  return GestureDetector(
    onTap: () {
      FocusScope.of(viewService.context).unfocus();
    },
    child: Container(
      color: Colors.black,
      child: Stack(
        children: [
          segmentContainer,
          ConfigDivider.create(),
          renderListView,
          pushButton,
        ],
      ),
    ),
  );
}

List<Widget> _containerRenderView(
    ConfigState state, Dispatch dispatch, ViewService viewService) {
  if (state.configData.selectSegmentIndex == 0) {
    return _parametersRenderContainer(state, dispatch, viewService);
  } else {
    return _featuresRenderContainer(state, dispatch, viewService);
  }
}

List<Widget> _parametersRenderContainer(
    ConfigState state, Dispatch dispatch, ViewService viewService) {
  List<Widget> renderView = [];

  if (state.pageType == PushConfigPageType.basic) {
    renderView.addAll([
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_live_push_mode,
        defaultValue: ConfigItemValue.intToBool(state.configData.livePushMode),
        action: (value) {
          int livePushMode = ConfigItemValue.boolToInt(value);
          dispatch(ConfigActionCreator.setLivePushMode(livePushMode));
        },
      ),
    ]);
  }

  renderView.addAll([
    ConfigCell(
      type: ConfigCellType.header,
      title: AppLocalizations.of(viewService.context)!.config_resolution_label,
    ),
    ConfigCell(
      type: ConfigCellType.slider,
      title: state.configData.resolution.stringV,
      defaultValue: state.configData.resolution.intV / 7.0,
      action: (value) {
        ConfigItemValue resolution =
            ConfigCellCommon.getResolutionDataAtSliderValue(value);
        dispatch(ConfigActionCreator.setResolution(resolution));
      },
    ),
  ]);

  if (state.pageType == PushConfigPageType.basic) {
    renderView.addAll([
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_auto_bitrate,
        defaultValue: state.configData.enableAutoBitrate,
        action: (value) {
          dispatch(ConfigActionCreator.setEnableAutoBitrate(value));
        },
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_auto_resolution,
        defaultValue: state.configData.enableAutoResolution,
        action: (value) {
          dispatch(ConfigActionCreator.setEnableAutoResolution(value));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!
            .config_auto_resolution_tip,
      ),
    ]);
  }

  renderView.addAll([
    ConfigCell(
      type: ConfigCellType.switchSelect,
      title: AppLocalizations.of(viewService.context)!.config_advanced,
      defaultValue: state.configData.advanced,
      action: (value) {
        ConfigDataUpdatePayload payload =
            ConfigDataUpdatePayload.create(ConfigAction.openAdvanced, value);
        dispatch(ConfigActionCreator.updateConfigData(payload));
      },
    ),
  ]);

  if (state.configData.advanced) {
    renderView.addAll(
        _parametersAdvancedOpenRenderContainer(state, dispatch, viewService));
  }

  return renderView;
}

List<Widget> _parametersAdvancedOpenRenderContainer(
    ConfigState state, Dispatch dispatch, ViewService viewService) {
  List<Widget> renderView = [];

  if (state.pageType == PushConfigPageType.basic) {
    renderView.addAll([
      ConfigCell(
        type: ConfigCellType.pickerSelect,
        title:
            AppLocalizations.of(viewService.context)!.config_quality_mode_label,
        defaultValue: state.configData.qualityMode.stringV,
        pickerPanelTexts: qualityModeItems.values.toList(),
        action: (value) {
          dispatch(ConfigActionCreator.setQualityMode(value.index));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!
            .config_quality_mode_label_tip,
      ),
      ConfigCell(
        type: ConfigCellType.input,
        title: AppLocalizations.of(viewService.context)!.config_target_bitrate,
        defaultValue: state.configData.targetVideoBitrate.toString(),
        infoUnit: "/Kbps",
        enable: state.configData.qualityMode.intV == 2,
        action: (value) {
          int intV = int.parse(value);
          dispatch(ConfigActionCreator.setTargetVideoBitrate(intV));
        },
      ),
      ConfigCell(
        type: ConfigCellType.input,
        title: AppLocalizations.of(viewService.context)!.config_min_bitrate,
        defaultValue: state.configData.minVideoBitrate.toString(),
        infoUnit: "/Kbps",
        enable: state.configData.qualityMode.intV == 2,
        action: (value) {
          int intV = int.parse(value);
          dispatch(ConfigActionCreator.setMinVideoBitrate(intV));
        },
      ),
      ConfigCell(
        type: ConfigCellType.input,
        title: AppLocalizations.of(viewService.context)!.config_initial_bitrate,
        defaultValue: state.configData.initialVideoBitrate.toString(),
        infoUnit: "/Kbps",
        enable: state.configData.qualityMode.intV == 2,
        action: (value) {
          int intV = int.parse(value);
          dispatch(ConfigActionCreator.setInitialVideoBitrate(intV));
        },
      ),
      ConfigCell(
        type: ConfigCellType.input,
        title: AppLocalizations.of(viewService.context)!.config_audio_bitrate,
        defaultValue: state.configData.audioBitrate.toString(),
        infoUnit: "/Kbps",
        action: (value) {
          int intV = int.parse(value);
          dispatch(ConfigActionCreator.setAudioBitrate(intV));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!.config_min_fps,
      ),
      ConfigCell(
        type: ConfigCellType.slider,
        title: state.configData.minFps.stringV.toString(),
        defaultValue: state.configData.minFps.intV / 7.0,
        infoUnit: "/fps",
        action: (value) {
          ConfigItemValue minFps = ConfigCellCommon.getFpsDataAtSliderValue(
              ConfigItemType.minFps, value);
          dispatch(ConfigActionCreator.setMinFps(minFps));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!.config_captrue_fps,
      ),
      ConfigCell(
        type: ConfigCellType.slider,
        title: state.configData.fps.stringV.toString(),
        defaultValue: state.configData.fps.intV / 7.0,
        infoUnit: "/fps",
        action: (value) {
          ConfigItemValue fps = ConfigCellCommon.getFpsDataAtSliderValue(
              ConfigItemType.fps, value);
          dispatch(ConfigActionCreator.setFps(fps));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title:
            AppLocalizations.of(viewService.context)!.config_audio_sample_rate,
      ),
      ConfigCell(
        type: ConfigCellType.slider,
        title: state.configData.audioSampleRate.stringV.toString(),
        defaultValue: state.configData.audioSampleRate.intV / 4.0,
        infoUnit: "/kHz",
        action: (value) {
          ConfigItemValue audioSampleRate =
              ConfigCellCommon.getAudioSampleRateDataAtSliderValue(value);
          dispatch(ConfigActionCreator.setAudioSampleRate(audioSampleRate));
        },
      ),
    ]);
  }

  renderView.addAll([
    ConfigCell(
      type: ConfigCellType.header,
      title: AppLocalizations.of(viewService.context)!.config_gop_size,
    ),
    ConfigCell(
      type: ConfigCellType.slider,
      title: state.configData.videoEncodeGop.toString(),
      defaultValue: state.configData.videoEncodeGop / 5.0,
      infoUnit: "/s",
      action: (value) {
        int videoEncodeGop =
            ConfigCellCommon.getVideoEncodeGopDataAtSliderValue(value);
        dispatch(ConfigActionCreator.setVideoEncodeGop(videoEncodeGop));
      },
    ),
  ]);

  if (state.pageType == PushConfigPageType.basic) {
    renderView.addAll([
      ConfigCell(
        type: ConfigCellType.pickerSelect,
        title: AppLocalizations.of(viewService.context)!.config_audio_profile,
        pickerPanelTexts: AudioEncoderProfileData.items.values.toList(),
        defaultValue: state.configData.audioEncoderProfile.stringV,
        action: (value) {
          dispatch(ConfigActionCreator.setAudioEncoderProfile(value.index));
        },
      ),
      ConfigCell(
        type: ConfigCellType.pickerSelect,
        title: AppLocalizations.of(viewService.context)!.config_sound_track,
        pickerPanelTexts: audioChannelItems.values.toList(),
        defaultValue: state.configData.audioChannel.stringV,
        action: (value) {
          dispatch(ConfigActionCreator.setAudioChannel(value.index));
        },
      ),
    ]);
  }

  renderView.addAll([
    ConfigCell(
      type: ConfigCellType.switchSelect,
      title: AppLocalizations.of(viewService.context)!
          .config_audio_hardware_encode,
      defaultValue: ConfigItemValue.intToBool(
          state.configData.audioEncoderMode == 0 ? 1 : 0),
      action: (value) {
        int audioEncoderMode =
            ConfigItemValue.boolToInt(value == false ? true : false);
        dispatch(ConfigActionCreator.setAudioEncoderMode(audioEncoderMode));
      },
    ),
    ConfigCell(
      type: ConfigCellType.switchSelect,
      title: AppLocalizations.of(viewService.context)!
          .config_video_hardware_encode,
      defaultValue: ConfigItemValue.intToBool(
          state.configData.videoEncoderMode == 0 ? 1 : 0),
      action: (value) {
        int videoEncoderMode =
            ConfigItemValue.boolToInt(value == false ? true : false);
        dispatch(ConfigActionCreator.setVideoEncoderMode(videoEncoderMode));
      },
    ),
  ]);

  if (state.configData.videoEncoderMode == 0) {
    renderView.addAll([
      ConfigCell(
        type: ConfigCellType.pickerSelect,
        title: AppLocalizations.of(viewService.context)!
            .config_video_hardware_encode_codec,
        pickerPanelTexts: videoHardEncoderCodecItems.values.toList(),
        defaultValue: state.configData.videoHardEncoderCodec.stringV,
        action: (value) {
          dispatch(ConfigActionCreator.setVideoHardEncoderCodec(value.index));
        },
      ),
    ]);

    if (state.pageType == PushConfigPageType.basic) {
      renderView.addAll([
        ConfigCell(
          type: ConfigCellType.switchSelect,
          title: AppLocalizations.of(viewService.context)!.config_BFrame,
          defaultValue: state.configData.openBFrame,
          action: (value) {
            dispatch(ConfigActionCreator.setOpenBFrame(value));
          },
        ),
      ]);
    }
  }

  return renderView;
}

List<Widget> _featuresRenderContainer(
    ConfigState state, Dispatch dispatch, ViewService viewService) {
  List<Widget> renderView = [];

  if (state.pageType == PushConfigPageType.basic) {
    renderView.addAll([
      ConfigCell(
        type: ConfigCellType.pickerSelect,
        title: AppLocalizations.of(viewService.context)!.config_landscape_model,
        defaultValue: state.configData.orientation.stringV,
        pickerPanelTexts: orientationItems.values.toList(),
        action: (value) {
          dispatch(ConfigActionCreator.setOrientation(value.index));
        },
      ),
      ConfigCell(
        type: ConfigCellType.pickerSelect,
        title: AppLocalizations.of(viewService.context)!
            .config_preview_display_mode,
        defaultValue: state.configData.previewDisplayMode.stringV,
        pickerPanelTexts: previewDisplayModeItems.values.toList(),
        action: (value) {
          dispatch(ConfigActionCreator.setPreviewDisplayMode(value.index));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title:
            AppLocalizations.of(viewService.context)!.config_auto_reconnection,
      ),
      ConfigCell(
        type: ConfigCellType.input,
        title:
            AppLocalizations.of(viewService.context)!.config_reconnect_duration,
        defaultValue:
            state.configData.connectRetryInterval.truncate().toString(),
        infoUnit: "/Kbps",
        action: (value) {
          double floatV = double.parse(value);
          dispatch(ConfigActionCreator.setConnectRetryInterval(floatV));
        },
      ),
      ConfigCell(
        type: ConfigCellType.input,
        title: AppLocalizations.of(viewService.context)!.config_reconnect_times,
        defaultValue: state.configData.connectRetryCount.toString(),
        infoUnit: "/" +
            AppLocalizations.of(viewService.context)!
                .config_reconnect_times_unit,
        action: (value) {
          int intV = int.parse(value);
          dispatch(ConfigActionCreator.setConnectRetryCount(intV));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!.config_mirroring,
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_push_mirror,
        defaultValue: state.configData.pushMirror,
        action: (value) {
          dispatch(ConfigActionCreator.setPushMirror(value));
        },
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_preview_mirror,
        defaultValue: state.configData.previewMirror,
        action: (value) {
          dispatch(ConfigActionCreator.setPreviewMirror(value));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!.config_camera_control,
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_front_camera,
        defaultValue: ConfigItemValue.intToBool(state.configData.cameraType),
        action: (value) {
          int cameraType = ConfigItemValue.boolToInt(value);
          dispatch(ConfigActionCreator.setCameraType(cameraType));
        },
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_auto_focus,
        defaultValue: state.configData.autoFocus,
        action: (value) {
          dispatch(ConfigActionCreator.setAutoFocus(value));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!.config_beauty_switch,
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_beauty_button,
        defaultValue: state.configData.beautyOn,
        action: (value) {
          dispatch(ConfigActionCreator.setBeautyOn(value));
        },
      ),
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!
            .config_ingest_standby_stream,
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title:
            AppLocalizations.of(viewService.context)!.config_background_image,
        defaultValue: state.configData.openPauseImg,
        action: (value) {
          dispatch(ConfigActionCreator.setPauseImg(value));
        },
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title:
            AppLocalizations.of(viewService.context)!.config_network_weak_image,
        defaultValue: state.configData.openNetworkPoorImg,
        action: (value) {
          dispatch(ConfigActionCreator.setNetworkPoorImg(value));
        },
      ),
    ]);
  }

  renderView.addAll([
    ConfigCell(
      type: ConfigCellType.pickerSelect,
      title: AppLocalizations.of(viewService.context)!.config_push_streaming,
      defaultValue: state.configData.pushStreamingMode.stringV,
      pickerPanelTexts: pushStreamingModeItems.values.toList(),
      action: (value) {
        dispatch(ConfigActionCreator.setPushStreamingMode(value.index));
      },
    ),
  ]);

  if (state.pageType == PushConfigPageType.basic) {
    renderView.addAll([
      ConfigCell(
        type: ConfigCellType.header,
        title: AppLocalizations.of(viewService.context)!.config_other_features,
      ),
      ConfigCell(
        type: ConfigCellType.switchSelect,
        title: AppLocalizations.of(viewService.context)!.config_local_logs,
        defaultValue: state.configData.openLog,
        action: (value) {
          dispatch(ConfigActionCreator.openLog(value));
        },
      ),
    ]);
  }

  return renderView;
}
